#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.


"""
Example Airflow DAG for Google Vertex AI service testing Pipeline Job operations.
"""

from __future__ import annotations

import os
from datetime import datetime

from google.cloud.aiplatform import schema
from google.protobuf.json_format import ParseDict
from google.protobuf.struct_pb2 import Value

from airflow.models.dag import DAG
from airflow.providers.google.cloud.operators.gcs import (
    GCSCreateBucketOperator,
    GCSDeleteBucketOperator,
    GCSDeleteObjectsOperator,
    GCSListObjectsOperator,
    GCSSynchronizeBucketsOperator,
)
from airflow.providers.google.cloud.operators.vertex_ai.dataset import CreateDatasetOperator
from airflow.providers.google.cloud.operators.vertex_ai.pipeline_job import (
    DeletePipelineJobOperator,
    GetPipelineJobOperator,
    ListPipelineJobOperator,
    RunPipelineJobOperator,
)

try:
    from airflow.sdk import TriggerRule
except ImportError:
    # Compatibility for Airflow < 3.1
    from airflow.utils.trigger_rule import TriggerRule  # type: ignore[no-redef,attr-defined]

ENV_ID = os.environ.get("SYSTEM_TESTS_ENV_ID", "default")
PROJECT_ID = os.environ.get("SYSTEM_TESTS_GCP_PROJECT", "default")
DAG_ID = "vertex_ai_pipeline_job_operations"
REGION = "us-central1"
DISPLAY_NAME = f"pipeline-job-{ENV_ID}"

RESOURCE_DATA_BUCKET = "airflow-system-tests-resources"
DATA_SAMPLE_GCS_BUCKET_NAME = f"bucket_{DAG_ID}_{ENV_ID}".replace("_", "-")
TEMPLATE_PATH = "https://us-kfp.pkg.dev/ml-pipeline/google-cloud-registry/get-vertex-dataset/sha256:f4eb4a2b0aab482c487c1cd62b3c735baaf914be8fa8c4687c06077c1d815a5d"
OUTPUT_BUCKET = f"gs://{DATA_SAMPLE_GCS_BUCKET_NAME}"

PARAMETER_VALUES = {
    "dataset_resource_name": f"projects/{PROJECT_ID}/locations/{REGION}/datasets/tabular-dataset-{ENV_ID}",
}

DATA_SAMPLE_GCS_OBJECT_NAME = "vertex-ai/california_housing_train.csv"
TABULAR_DATASET = {
    "display_name": f"tabular-dataset-{ENV_ID}",
    "metadata_schema_uri": schema.dataset.metadata.tabular,
    "metadata": ParseDict(
        {
            "input_config": {
                "gcs_source": {"uri": [f"gs://{DATA_SAMPLE_GCS_BUCKET_NAME}/{DATA_SAMPLE_GCS_OBJECT_NAME}"]}
            }
        },
        Value(),
    ),
}


with DAG(
    DAG_ID,
    schedule="@once",
    start_date=datetime(2021, 1, 1),
    catchup=False,
    tags=["example", "vertex_ai", "pipeline_job"],
) as dag:
    create_bucket = GCSCreateBucketOperator(
        task_id="create_bucket",
        bucket_name=DATA_SAMPLE_GCS_BUCKET_NAME,
        storage_class="REGIONAL",
        location=REGION,
    )

    move_dataset_files = GCSSynchronizeBucketsOperator(
        task_id="move_dataset_files_to_bucket",
        source_bucket=RESOURCE_DATA_BUCKET,
        source_object="vertex-ai/california-housing-data",
        destination_bucket=DATA_SAMPLE_GCS_BUCKET_NAME,
        destination_object="vertex-ai",
        recursive=True,
    )

    create_dataset = CreateDatasetOperator(
        task_id="tabular_dataset",
        dataset=TABULAR_DATASET,
        region=REGION,
        project_id=PROJECT_ID,
    )

    # [START how_to_cloud_vertex_ai_run_pipeline_job_operator]
    run_pipeline_job = RunPipelineJobOperator(
        task_id="run_pipeline_job",
        display_name=DISPLAY_NAME,
        template_path=TEMPLATE_PATH,
        parameter_values=PARAMETER_VALUES,
        region=REGION,
        project_id=PROJECT_ID,
    )
    # [END how_to_cloud_vertex_ai_run_pipeline_job_operator]

    # [START how_to_cloud_vertex_ai_get_pipeline_job_operator]
    get_pipeline_job = GetPipelineJobOperator(
        task_id="get_pipeline_job",
        project_id=PROJECT_ID,
        region=REGION,
        pipeline_job_id="{{ task_instance.xcom_pull(task_ids='run_pipeline_job', key='pipeline_job_id') }}",
    )
    # [END how_to_cloud_vertex_ai_get_pipeline_job_operator]

    # [START how_to_cloud_vertex_ai_delete_pipeline_job_operator]
    delete_pipeline_job = DeletePipelineJobOperator(
        task_id="delete_pipeline_job",
        project_id=PROJECT_ID,
        region=REGION,
        pipeline_job_id="{{ task_instance.xcom_pull(task_ids='run_pipeline_job', key='pipeline_job_id') }}",
        trigger_rule=TriggerRule.ALL_DONE,
    )
    # [END how_to_cloud_vertex_ai_delete_pipeline_job_operator]

    # [START how_to_cloud_vertex_ai_list_pipeline_job_operator]
    list_pipeline_job = ListPipelineJobOperator(
        task_id="list_pipeline_job",
        region=REGION,
        project_id=PROJECT_ID,
    )
    # [END how_to_cloud_vertex_ai_list_pipeline_job_operator]

    list_buckets = GCSListObjectsOperator(task_id="list_buckets", bucket=DATA_SAMPLE_GCS_BUCKET_NAME)

    delete_files = GCSDeleteObjectsOperator(
        task_id="delete_files", bucket_name=DATA_SAMPLE_GCS_BUCKET_NAME, objects=list_buckets.output
    )

    delete_bucket = GCSDeleteBucketOperator(
        task_id="delete_bucket",
        bucket_name=DATA_SAMPLE_GCS_BUCKET_NAME,
        trigger_rule=TriggerRule.ALL_DONE,
    )

    (
        # TEST SETUP
        create_bucket
        >> move_dataset_files
        >> create_dataset
        # TEST BODY
        >> run_pipeline_job
        >> get_pipeline_job
        >> delete_pipeline_job
        >> list_pipeline_job
        # TEST TEARDOWN
        >> list_buckets
        >> delete_files
        >> delete_bucket
    )

    # ### Everything below this line is not part of example ###
    # ### Just for system tests purpose ###
    from tests_common.test_utils.watcher import watcher

    # This test needs watcher in order to properly mark success/failure
    # when "tearDown" task with trigger rule is part of the DAG
    list(dag.tasks) >> watcher()

from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""This module contains a helper class to work with `google.api_core.operation.Operation` object."""

from __future__ import annotations

from typing import TYPE_CHECKING

from google.api_core.exceptions import GoogleAPICallError

from airflow.exceptions import AirflowException

if TYPE_CHECKING:
    from google.api_core.operation import Operation
    from google.api_core.retry import Retry
    from proto import Message


class OperationHelper:
    """Helper class to work with `operation.Operation` objects."""

    @staticmethod
    def wait_for_operation_result(
        operation: Operation,
        timeout: int | None = None,
        polling: Retry | None = None,
        retry: Retry | None = None,
    ) -> Message:
        """
        Wait for long-lasting operation result to be retrieved.

        For advance usage please check the docs on:
        :class:`google.api_core.future.polling.PollingFuture`
        :class:`google.api_core.retry.Retry`

        :param operation: The initial operation to get result from.
        :param timeout: How long (in seconds) to wait for the operation to complete.
            If None, wait indefinitely. Overrides polling.timeout if both specified.
        :param polling: How often and for how long to call polling RPC periodically.
        :param retry: How to retry the operation polling if error occurs.
        """
        try:
            return operation.result(timeout=timeout, polling=polling, retry=retry)
        except GoogleAPICallError as ex:
            raise AirflowException("Google API error on operation result call") from ex
        except Exception:
            error = operation.exception(timeout=timeout)
            raise AirflowException(error)

    def wait_for_operation(
        self,
        operation: Operation,
        timeout: float | int | None = None,
    ):
        """
        Legacy method name wrapper.

        Intended to use with existing hooks/operators, until the proper deprecation and replacement provided.
        """
        if isinstance(timeout, float):
            timeout = int(timeout)

        return self.wait_for_operation_result(operation=operation, timeout=timeout)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

"""
Example Airflow DAG for Google Cloud Dataflow YAML service.

Requirements:
    This test requires ``gcloud`` command (Google Cloud SDK) to be installed on the Airflow worker
    <https://cloud.google.com/sdk/docs/install>`__
"""

from __future__ import annotations

import os
from datetime import datetime

from airflow.models.dag import DAG
from airflow.providers.google.cloud.hooks.dataflow import DataflowJobStatus
from airflow.providers.google.cloud.operators.bigquery import (
    BigQueryCreateEmptyDatasetOperator,
    BigQueryCreateTableOperator,
    BigQueryDeleteDatasetOperator,
    BigQueryInsertJobOperator,
)
from airflow.providers.google.cloud.operators.dataflow import DataflowStartYamlJobOperator

try:
    from airflow.sdk import TriggerRule
except ImportError:
    # Compatibility for Airflow < 3.1
    from airflow.utils.trigger_rule import TriggerRule  # type: ignore[no-redef,attr-defined]

from system.google import DEFAULT_GCP_SYSTEM_TEST_PROJECT_ID

PROJECT_ID = os.environ.get("SYSTEM_TESTS_GCP_PROJECT") or DEFAULT_GCP_SYSTEM_TEST_PROJECT_ID
ENV_ID = os.environ.get("SYSTEM_TESTS_ENV_ID", "default")
DAG_ID = "dataflow_yaml"
REGION = "europe-west2"
DATAFLOW_YAML_JOB_NAME = f"{DAG_ID}_{ENV_ID}".replace("_", "-")
BQ_DATASET = f"{DAG_ID}_{ENV_ID}".replace("-", "_")
BQ_INPUT_TABLE = f"input_{DAG_ID}".replace("-", "_")
BQ_OUTPUT_TABLE = f"output_{DAG_ID}".replace("-", "_")
DATAFLOW_YAML_PIPELINE_FILE_URL = (
    "gs://airflow-system-tests-resources/dataflow/yaml/example_beam_yaml_bq.yaml"
)
SCHEMA = [
    {"name": "emp_name", "type": "STRING", "mode": "REQUIRED"},
    {"name": "salary", "type": "INTEGER", "mode": "NULLABLE"},
    {"name": "country", "type": "STRING", "mode": "NULLABLE"},
]

BQ_VARIABLES = {
    "project": PROJECT_ID,
    "dataset": BQ_DATASET,
    "input": BQ_INPUT_TABLE,
    "output": BQ_OUTPUT_TABLE,
}

BQ_VARIABLES_DEF = {
    "project": PROJECT_ID,
    "dataset": BQ_DATASET,
    "input": BQ_INPUT_TABLE,
    "output": f"{BQ_OUTPUT_TABLE}_def",
}

INSERT_ROWS_QUERY = (
    f"INSERT {BQ_DATASET}.{BQ_INPUT_TABLE} VALUES "
    "('John Doe', 900, 'USA'), "
    "('Alice Storm', 1200, 'Australia'),"
    "('Bob Max', 1000, 'Australia'),"
    "('Peter Jackson', 800, 'New Zealand'),"
    "('Hobby Doyle', 1100, 'USA'),"
    "('Terrance Phillips', 2222, 'Canada'),"
    "('Joe Schmoe', 1500, 'Canada'),"
    "('Dominique Levillaine', 2780, 'France');"
)


with DAG(
    dag_id=DAG_ID,
    start_date=datetime(2021, 1, 1),
    schedule="@once",
    catchup=False,
    tags=["example", "dataflow", "yaml"],
) as dag:
    create_bq_dataset = BigQueryCreateEmptyDatasetOperator(
        task_id="create_bq_dataset",
        dataset_id=BQ_DATASET,
        location=REGION,
    )

    create_bq_input_table = BigQueryCreateTableOperator(
        task_id="create_bq_input_table",
        dataset_id=BQ_DATASET,
        table_id=BQ_INPUT_TABLE,
        table_resource={
            "schema": {"fields": SCHEMA},
        },
    )

    insert_data_into_bq_table = BigQueryInsertJobOperator(
        task_id="insert_data_into_bq_table",
        configuration={
            "query": {
                "query": INSERT_ROWS_QUERY,
                "useLegacySql": False,
                "priority": "BATCH",
            }
        },
        location=REGION,
    )

    # [START howto_operator_dataflow_start_yaml_job]
    start_dataflow_yaml_job = DataflowStartYamlJobOperator(
        task_id="start_dataflow_yaml_job",
        job_name=DATAFLOW_YAML_JOB_NAME,
        yaml_pipeline_file=DATAFLOW_YAML_PIPELINE_FILE_URL,
        append_job_name=True,
        deferrable=False,
        region=REGION,
        project_id=PROJECT_ID,
        jinja_variables=BQ_VARIABLES,
    )
    # [END howto_operator_dataflow_start_yaml_job]

    # [START howto_operator_dataflow_start_yaml_job_def]
    start_dataflow_yaml_job_def = DataflowStartYamlJobOperator(
        task_id="start_dataflow_yaml_job_def",
        job_name=DATAFLOW_YAML_JOB_NAME,
        yaml_pipeline_file=DATAFLOW_YAML_PIPELINE_FILE_URL,
        append_job_name=True,
        deferrable=True,
        region=REGION,
        project_id=PROJECT_ID,
        jinja_variables=BQ_VARIABLES_DEF,
        expected_terminal_state=DataflowJobStatus.JOB_STATE_DONE,
    )
    # [END howto_operator_dataflow_start_yaml_job_def]

    delete_bq_dataset = BigQueryDeleteDatasetOperator(
        task_id="delete_bq_dataset",
        dataset_id=BQ_DATASET,
        delete_contents=True,
        trigger_rule=TriggerRule.ALL_DONE,
    )

    (
        # TEST SETUP
        create_bq_dataset
        >> create_bq_input_table
        >> insert_data_into_bq_table
        # TEST BODY
        >> [start_dataflow_yaml_job, start_dataflow_yaml_job_def]
        # TEST TEARDOWN
        >> delete_bq_dataset
    )

    from tests_common.test_utils.watcher import watcher

    # This test needs watcher in order to properly mark success/failure
    # when "tearDown" task with trigger rule is part of the DAG
    list(dag.tasks) >> watcher()

from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

import pytest
from google.cloud.logging_v2.types import (
    CreateSinkRequest,
    DeleteSinkRequest,
    GetSinkRequest,
    ListSinksRequest,
    LogSink,
    UpdateSinkRequest,
)

from airflow.providers.google.cloud.hooks.cloud_logging import CloudLoggingHook

from unit.google.cloud.utils.base_gcp_mock import mock_base_gcp_hook_default_project_id

BASE_STRING = "airflow.providers.google.common.hooks.base_google.{}"
CLOUDLOGGING_HOOK_CLIENT = "airflow.providers.google.cloud.hooks.cloud_logging.CloudLoggingHook.get_conn"

PROJECT_ID = "gcp-project-id"
SINK_NAME = "my-logs-sink"
UNIQUE_WRITER_IDENTITY = True
sink_config = {
    "name": SINK_NAME,
    "destination": "storage.googleapis.com/test-log-sink-af",
    "description": "Create with full sink_config",
    "filter": "severity>=INFO",
    "disabled": False,
    "exclusions": [
        {
            "name": "exclude-debug",
            "description": "Skip debug logs",
            "filter": "severity=DEBUG",
            "disabled": True,
        },
        {
            "name": "exclude-cloudsql",
            "description": "Skip CloudSQL logs",
            "filter": 'resource.type="cloudsql_database"',
            "disabled": False,
        },
    ],
}


GCP_CONN_ID = "google_cloud_default"


class TestCloudLoggingHook:
    def setup_method(self):
        with mock.patch(
            BASE_STRING.format("GoogleBaseHook.__init__"),
            new=mock_base_gcp_hook_default_project_id,
        ):
            self.hook = CloudLoggingHook(gcp_conn_id=GCP_CONN_ID)

    @mock.patch(
        "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.__init__",
        new=mock_base_gcp_hook_default_project_id,
    )
    @mock.patch("airflow.providers.google.cloud.hooks.cloud_logging.ConfigServiceV2Client")
    @mock.patch("airflow.providers.google.common.hooks.base_google.GoogleBaseHook.get_credentials")
    def test_get_conn(self, mock_get_credentials, mock_client_class):
        mock_credentials = mock.Mock()
        mock_get_credentials.return_value = mock_credentials

        hook = CloudLoggingHook(gcp_conn_id=GCP_CONN_ID)
        conn = hook.get_conn()

        mock_client_class.assert_called_once_with(credentials=mock_credentials, client_info=mock.ANY)
        assert conn == mock_client_class.return_value

    @mock.patch(CLOUDLOGGING_HOOK_CLIENT)
    def test_create_sink(self, mock_config_client):
        sink = LogSink(**sink_config)
        expected_request = CreateSinkRequest(
            parent=f"projects/{PROJECT_ID}", sink=sink, unique_writer_identity=UNIQUE_WRITER_IDENTITY
        )

        self.hook.create_sink(
            sink=sink,
            project_id=PROJECT_ID,
            unique_writer_identity=UNIQUE_WRITER_IDENTITY,
        )

        mock_config_client.return_value.create_sink.assert_called_once_with(request=expected_request)

    @mock.patch(CLOUDLOGGING_HOOK_CLIENT)
    def test_get_sink(self, mock_config_client):
        expected_request = GetSinkRequest(sink_name=f"projects/{PROJECT_ID}/sinks/{SINK_NAME}")
        self.hook.get_sink(sink_name=SINK_NAME, project_id=PROJECT_ID)
        mock_config_client.return_value.get_sink.assert_called_once_with(request=expected_request)

    @mock.patch(CLOUDLOGGING_HOOK_CLIENT)
    def test_list_sinks(self, mock_config_client):
        expected_request = ListSinksRequest(parent=f"projects/{PROJECT_ID}")
        self.hook.list_sinks(project_id=PROJECT_ID)
        mock_config_client.return_value.list_sinks.assert_called_once_with(request=expected_request)

    @mock.patch(CLOUDLOGGING_HOOK_CLIENT)
    def test_delete_sink(self, mock_config_client):
        expected_request = DeleteSinkRequest(sink_name=f"projects/{PROJECT_ID}/sinks/{SINK_NAME}")
        self.hook.delete_sink(sink_name=SINK_NAME, project_id=PROJECT_ID)
        mock_config_client.return_value.delete_sink.assert_called_once_with(request=expected_request)

    @mock.patch(CLOUDLOGGING_HOOK_CLIENT)
    def test_update_sink_success(self, mock_config_client):
        sink_config = {
            "destination": f"bigquery.googleapis.com/projects/{PROJECT_ID}/datasets/your_dataset",
            "bigquery_options": {"use_partitioned_tables": True},
        }
        update_mask = {"paths": ["destination", "bigquery_options"]}

        expected_request = UpdateSinkRequest(
            sink_name=f"projects/{PROJECT_ID}/sinks/{SINK_NAME}",
            sink=sink_config,
            update_mask=update_mask,
            unique_writer_identity=UNIQUE_WRITER_IDENTITY,
        )

        self.hook.update_sink(
            sink_name=SINK_NAME,
            sink=sink_config,
            update_mask=update_mask,
            unique_writer_identity=UNIQUE_WRITER_IDENTITY,
            project_id=PROJECT_ID,
        )

        mock_config_client.return_value.update_sink.assert_called_once_with(request=expected_request)

    @mock.patch(CLOUDLOGGING_HOOK_CLIENT)
    def test_create_sink_dict_input(self, mock_config_client):
        expected_sink = LogSink(**sink_config)
        expected_request = CreateSinkRequest(
            parent=f"projects/{PROJECT_ID}", sink=expected_sink, unique_writer_identity=UNIQUE_WRITER_IDENTITY
        )

        self.hook.create_sink(
            sink=sink_config, unique_writer_identity=UNIQUE_WRITER_IDENTITY, project_id=PROJECT_ID
        )

        mock_config_client.return_value.create_sink.assert_called_once_with(request=expected_request)

    def test_update_sink_invalid_dict_format(self):
        with pytest.raises(ValueError, match="Unknown field for LogSink: invalid_key"):
            self.hook.update_sink(
                sink_name=SINK_NAME,
                sink={"invalid_key": "value"},
                update_mask={"paths": ["invalid_key"]},
                unique_writer_identity=UNIQUE_WRITER_IDENTITY,
                project_id=PROJECT_ID,
            )

    @mock.patch(CLOUDLOGGING_HOOK_CLIENT)
    def test_update_sink_failure(self, mock_config_client):
        updated_sink = LogSink(name=SINK_NAME, destination="storage.googleapis.com/new-bucket")
        updated_mask = {"paths": ["name", "destination"]}

        mock_config_client.return_value.update_sink.side_effect = Exception("Permission denied")

        with pytest.raises(Exception, match="Permission denied"):
            self.hook.update_sink(
                sink_name=SINK_NAME,
                sink=updated_sink,
                update_mask=updated_mask,
                unique_writer_identity=UNIQUE_WRITER_IDENTITY,
                project_id=PROJECT_ID,
            )

        mock_config_client.return_value.update_sink.assert_called_once()

    @mock.patch(CLOUDLOGGING_HOOK_CLIENT)
    def test_list_sinks_empty(self, mock_config_client):
        mock_config_client.return_value.list_sinks.return_value = []

        sinks = self.hook.list_sinks(project_id=PROJECT_ID)

        assert sinks == []
        mock_config_client.return_value.list_sinks.assert_called_once()



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""This module contains Google Analytics 4 (GA4) operators."""

from __future__ import annotations

from collections.abc import Sequence
from typing import TYPE_CHECKING, Any

from google.analytics.admin_v1beta import (
    Account,
    DataStream,
    GoogleAdsLink,
    Property,
)
from google.api_core.gapic_v1.method import DEFAULT, _MethodDefault

from airflow.exceptions import AirflowNotFoundException
from airflow.providers.google.cloud.operators.cloud_base import GoogleCloudBaseOperator
from airflow.providers.google.marketing_platform.hooks.analytics_admin import GoogleAnalyticsAdminHook
from airflow.providers.google.marketing_platform.links.analytics_admin import GoogleAnalyticsPropertyLink

if TYPE_CHECKING:
    from google.api_core.retry import Retry
    from google.protobuf.message import Message

    from airflow.providers.common.compat.sdk import Context


class GoogleAnalyticsAdminListAccountsOperator(GoogleCloudBaseOperator):
    """
    Lists all accounts to which the user has access.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:GoogleAnalyticsAdminListAccountsOperator`

    :param page_size: Optional, number of results to return in the list.
    :param page_token: Optional. The next_page_token value returned from a previous List request, if any.
    :param show_deleted: Optional. Whether to include soft-deleted (ie: "trashed") Accounts in the results.
    :param retry: Optional, a retry object used  to retry requests. If `None` is specified, requests
            will not be retried.
    :param timeout: Optional. The timeout for this request.
    :param metadata: Optional. Strings which should be sent along with the request as metadata.
    :param gcp_conn_id: The connection ID to use when fetching connection info.
    :param impersonation_chain: Optional. Service account to impersonate using short-term
        credentials, or chained list of accounts required to get the access_token
        of the last account in the list, which will be impersonated in the request.
        If set as a string, the account must grant the originating account
        the Service Account Token Creator IAM role.
        If set as a sequence, the identities from the list must grant
        Service Account Token Creator IAM role to the directly preceding identity, with first
        account from the list granting this role to the originating account (templated).
    """

    template_fields: Sequence[str] = (
        "gcp_conn_id",
        "impersonation_chain",
        "page_size",
        "page_token",
    )

    def __init__(
        self,
        *,
        page_size: int | None = None,
        page_token: str | None = None,
        show_deleted: bool | None = None,
        retry: Retry | _MethodDefault = DEFAULT,
        timeout: float | None = None,
        metadata: Sequence[tuple[str, str]] = (),
        gcp_conn_id: str = "google_cloud_default",
        impersonation_chain: str | Sequence[str] | None = None,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.page_size = page_size
        self.page_token = page_token
        self.show_deleted = show_deleted
        self.retry = retry
        self.timeout = timeout
        self.metadata = metadata
        self.gcp_conn_id = gcp_conn_id
        self.impersonation_chain = impersonation_chain

    def execute(
        self,
        context: Context,
    ) -> Sequence[Message]:
        hook = GoogleAnalyticsAdminHook(
            gcp_conn_id=self.gcp_conn_id,
            impersonation_chain=self.impersonation_chain,
        )
        self.log.info(
            "Requesting list of Google Analytics accounts. Page size: %s, page token: %s",
            self.page_size,
            self.page_token,
        )
        accounts = hook.list_accounts(
            page_size=self.page_size,
            page_token=self.page_token,
            show_deleted=self.show_deleted,
            retry=self.retry,
            timeout=self.timeout,
            metadata=self.metadata,
        )
        accounts_list: Sequence[Message] = [Account.to_dict(item) for item in accounts]
        n = len(accounts_list)
        self.log.info("Successful request. Retrieved %s item%s.", n, "s" if n > 1 else "")
        return accounts_list


class GoogleAnalyticsAdminCreatePropertyOperator(GoogleCloudBaseOperator):
    """
    Creates property.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:GoogleAnalyticsAdminCreatePropertyOperator`

    :param analytics_property: The property to create. Note: the supplied property must specify its parent.
        For more details see: https://developers.google.com/analytics/devguides/config/admin/v1/rest/v1beta/properties#Property
    :param retry: Optional, a retry object used  to retry requests. If `None` is specified, requests
        will not be retried.
    :param timeout: Optional. The timeout for this request.
    :param metadata: Optional. Strings which should be sent along with the request as metadata.
    :param gcp_conn_id: The connection ID to use when fetching connection info.
    :param impersonation_chain: Optional. Service account to impersonate using short-term
        credentials, or chained list of accounts required to get the access_token
        of the last account in the list, which will be impersonated in the request.
        If set as a string, the account must grant the originating account
        the Service Account Token Creator IAM role.
        If set as a sequence, the identities from the list must grant
        Service Account Token Creator IAM role to the directly preceding identity, with first
        account from the list granting this role to the originating account (templated).
    """

    template_fields: Sequence[str] = (
        "gcp_conn_id",
        "impersonation_chain",
        "analytics_property",
    )
    operator_extra_links = (GoogleAnalyticsPropertyLink(),)

    def __init__(
        self,
        *,
        analytics_property: Property | dict[str, Any],
        retry: Retry | _MethodDefault = DEFAULT,
        timeout: float | None = None,
        metadata: Sequence[tuple[str, str]] = (),
        gcp_conn_id: str = "google_cloud_default",
        impersonation_chain: str | Sequence[str] | None = None,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.analytics_property = analytics_property
        self.retry = retry
        self.timeout = timeout
        self.metadata = metadata
        self.gcp_conn_id = gcp_conn_id
        self.impersonation_chain = impersonation_chain

    def execute(
        self,
        context: Context,
    ) -> Message:
        hook = GoogleAnalyticsAdminHook(
            gcp_conn_id=self.gcp_conn_id,
            impersonation_chain=self.impersonation_chain,
        )
        self.log.info("Creating a Google Analytics property.")
        prop = hook.create_property(
            analytics_property=self.analytics_property,
            retry=self.retry,
            timeout=self.timeout,
            metadata=self.metadata,
        )
        self.log.info("The Google Analytics property %s was created successfully.", prop.name)
        GoogleAnalyticsPropertyLink.persist(
            context=context,
            property_id=prop.name.lstrip("properties/"),
        )

        return Property.to_dict(prop)


class GoogleAnalyticsAdminDeletePropertyOperator(GoogleCloudBaseOperator):
    """
    Soft-delete property.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:GoogleAnalyticsAdminDeletePropertyOperator`

    :param property_id: The id of the Property to soft-delete.
    :param retry: Optional, a retry object used  to retry requests. If `None` is specified, requests
        will not be retried.
    :param timeout: Optional. The timeout for this request.
    :param metadata: Optional. Strings which should be sent along with the request as metadata.
    :param gcp_conn_id: The connection ID to use when fetching connection info.
    :param impersonation_chain: Optional. Service account to impersonate using short-term
        credentials, or chained list of accounts required to get the access_token
        of the last account in the list, which will be impersonated in the request.
        If set as a string, the account must grant the originating account
        the Service Account Token Creator IAM role.
        If set as a sequence, the identities from the list must grant
        Service Account Token Creator IAM role to the directly preceding identity, with first
        account from the list granting this role to the originating account (templated).
    """

    template_fields: Sequence[str] = (
        "gcp_conn_id",
        "impersonation_chain",
        "property_id",
    )

    def __init__(
        self,
        *,
        property_id: str,
        retry: Retry | _MethodDefault = DEFAULT,
        timeout: float | None = None,
        metadata: Sequence[tuple[str, str]] = (),
        gcp_conn_id: str = "google_cloud_default",
        impersonation_chain: str | Sequence[str] | None = None,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.property_id = property_id
        self.retry = retry
        self.timeout = timeout
        self.metadata = metadata
        self.gcp_conn_id = gcp_conn_id
        self.impersonation_chain = impersonation_chain

    def execute(
        self,
        context: Context,
    ) -> Message:
        hook = GoogleAnalyticsAdminHook(
            gcp_conn_id=self.gcp_conn_id,
            impersonation_chain=self.impersonation_chain,
        )
        self.log.info("Deleting a Google Analytics property.")
        prop = hook.delete_property(
            property_id=self.property_id,
            retry=self.retry,
            timeout=self.timeout,
            metadata=self.metadata,
        )
        self.log.info("The Google Analytics property %s was soft-deleted successfully.", prop.name)
        return Property.to_dict(prop)


class GoogleAnalyticsAdminCreateDataStreamOperator(GoogleCloudBaseOperator):
    """
    Creates Data stream.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:GoogleAnalyticsAdminCreateDataStreamOperator`

    :param property_id: ID of the parent property for the data stream.
    :param data_stream: The data stream to create.
        For more details see: https://developers.google.com/analytics/devguides/config/admin/v1/rest/v1beta/properties.dataStreams#DataStream
    :param retry: Optional, a retry object used  to retry requests. If `None` is specified, requests
        will not be retried.
    :param timeout: Optional. The timeout for this request.
    :param metadata: Optional. Strings which should be sent along with the request as metadata.
    :param gcp_conn_id: The connection ID to use when fetching connection info.
    :param impersonation_chain: Optional. Service account to impersonate using short-term
        credentials, or chained list of accounts required to get the access_token
        of the last account in the list, which will be impersonated in the request.
        If set as a string, the account must grant the originating account
        the Service Account Token Creator IAM role.
        If set as a sequence, the identities from the list must grant
        Service Account Token Creator IAM role to the directly preceding identity, with first
        account from the list granting this role to the originating account (templated).
    """

    template_fields: Sequence[str] = (
        "gcp_conn_id",
        "impersonation_chain",
        "property_id",
        "data_stream",
    )

    def __init__(
        self,
        *,
        property_id: str,
        data_stream: DataStream | dict,
        retry: Retry | _MethodDefault = DEFAULT,
        timeout: float | None = None,
        metadata: Sequence[tuple[str, str]] = (),
        gcp_conn_id: str = "google_cloud_default",
        impersonation_chain: str | Sequence[str] | None = None,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.property_id = property_id
        self.data_stream = data_stream
        self.retry = retry
        self.timeout = timeout
        self.metadata = metadata
        self.gcp_conn_id = gcp_conn_id
        self.impersonation_chain = impersonation_chain

    def execute(
        self,
        context: Context,
    ) -> Message:
        hook = GoogleAnalyticsAdminHook(
            gcp_conn_id=self.gcp_conn_id,
            impersonation_chain=self.impersonation_chain,
        )
        self.log.info("Creating a Google Analytics data stream.")
        data_stream = hook.create_data_stream(
            property_id=self.property_id,
            data_stream=self.data_stream,
            retry=self.retry,
            timeout=self.timeout,
            metadata=self.metadata,
        )
        self.log.info("The Google Analytics data stream %s was created successfully.", data_stream.name)
        return DataStream.to_dict(data_stream)


class GoogleAnalyticsAdminDeleteDataStreamOperator(GoogleCloudBaseOperator):
    """
    Deletes Data stream.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:GoogleAnalyticsAdminDeleteDataStreamOperator`

    :param property_id: ID of the property which is parent for the data stream.
    :param data_stream_id: ID of the data stream to delete.
    :param retry: Optional, a retry object used  to retry requests. If `None` is specified, requests
        will not be retried.
    :param timeout: Optional. The timeout for this request.
    :param metadata: Optional. Strings which should be sent along with the request as metadata.
    :param gcp_conn_id: The connection ID to use when fetching connection info.
    :param impersonation_chain: Optional. Service account to impersonate using short-term
        credentials, or chained list of accounts required to get the access_token
        of the last account in the list, which will be impersonated in the request.
        If set as a string, the account must grant the originating account
        the Service Account Token Creator IAM role.
        If set as a sequence, the identities from the list must grant
        Service Account Token Creator IAM role to the directly preceding identity, with first
        account from the list granting this role to the originating account (templated).
    """

    template_fields: Sequence[str] = (
        "gcp_conn_id",
        "impersonation_chain",
        "property_id",
        "data_stream_id",
    )

    def __init__(
        self,
        *,
        property_id: str,
        data_stream_id: str,
        retry: Retry | _MethodDefault = DEFAULT,
        timeout: float | None = None,
        metadata: Sequence[tuple[str, str]] = (),
        gcp_conn_id: str = "google_cloud_default",
        impersonation_chain: str | Sequence[str] | None = None,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.property_id = property_id
        self.data_stream_id = data_stream_id
        self.retry = retry
        self.timeout = timeout
        self.metadata = metadata
        self.gcp_conn_id = gcp_conn_id
        self.impersonation_chain = impersonation_chain

    def execute(
        self,
        context: Context,
    ) -> None:
        hook = GoogleAnalyticsAdminHook(
            gcp_conn_id=self.gcp_conn_id,
            impersonation_chain=self.impersonation_chain,
        )
        self.log.info("Deleting a Google Analytics data stream (id %s).", self.data_stream_id)
        hook.delete_data_stream(
            property_id=self.property_id,
            data_stream_id=self.data_stream_id,
            retry=self.retry,
            timeout=self.timeout,
            metadata=self.metadata,
        )
        self.log.info("The Google Analytics data stream was deleted successfully.")
        return None


class GoogleAnalyticsAdminListGoogleAdsLinksOperator(GoogleCloudBaseOperator):
    """
    Lists all Google Ads links associated with a given property.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:GoogleAnalyticsAdminListGoogleAdsLinksOperator`

    :param property_id: ID of the parent property.
    :param page_size: Optional, number of results to return in the list.
    :param page_token: Optional. The next_page_token value returned from a previous List request, if any.
    :param retry: Optional, a retry object used  to retry requests. If `None` is specified, requests
        will not be retried.
    :param timeout: Optional. The timeout for this request.
    :param metadata: Optional. Strings which should be sent along with the request as metadata.
    :param gcp_conn_id: The connection ID to use when fetching connection info.
    :param impersonation_chain: Optional. Service account to impersonate using short-term
        credentials, or chained list of accounts required to get the access_token
        of the last account in the list, which will be impersonated in the request.
        If set as a string, the account must grant the originating account
        the Service Account Token Creator IAM role.
        If set as a sequence, the identities from the list must grant
        Service Account Token Creator IAM role to the directly preceding identity, with first
        account from the list granting this role to the originating account (templated).
    """

    template_fields: Sequence[str] = (
        "gcp_conn_id",
        "impersonation_chain",
        "property_id",
        "page_size",
        "page_token",
    )

    def __init__(
        self,
        *,
        property_id: str,
        page_size: int | None = None,
        page_token: str | None = None,
        retry: Retry | _MethodDefault = DEFAULT,
        timeout: float | None = None,
        metadata: Sequence[tuple[str, str]] = (),
        gcp_conn_id: str = "google_cloud_default",
        impersonation_chain: str | Sequence[str] | None = None,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.property_id = property_id
        self.page_size = page_size
        self.page_token = page_token
        self.retry = retry
        self.timeout = timeout
        self.metadata = metadata
        self.gcp_conn_id = gcp_conn_id
        self.impersonation_chain = impersonation_chain

    def execute(
        self,
        context: Context,
    ) -> Sequence[Message]:
        hook = GoogleAnalyticsAdminHook(
            gcp_conn_id=self.gcp_conn_id,
            impersonation_chain=self.impersonation_chain,
        )
        self.log.info(
            "Requesting list of Google Ads links accounts for the property_id %s, "
            "page size %s, page token %s",
            self.property_id,
            self.page_size,
            self.page_token,
        )
        google_ads_links = hook.list_google_ads_links(
            property_id=self.property_id,
            page_size=self.page_size,
            page_token=self.page_token,
            retry=self.retry,
            timeout=self.timeout,
            metadata=self.metadata,
        )
        ads_links_list: Sequence[Message] = [GoogleAdsLink.to_dict(item) for item in google_ads_links]
        n = len(ads_links_list)
        self.log.info("Successful request. Retrieved %s item%s.", n, "s" if n > 1 else "")
        return ads_links_list


class GoogleAnalyticsAdminGetGoogleAdsLinkOperator(GoogleCloudBaseOperator):
    """
    Gets a Google Ads link associated with a given property.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:GoogleAnalyticsAdminGetGoogleAdsLinkOperator`

    :param property_id: Parent property id.
    :param google_ads_link_id: Google Ads link id.
    :param retry: Optional, a retry object used  to retry requests. If `None` is specified, requests
        will not be retried.
    :param timeout: Optional. The timeout for this request.
    :param metadata: Optional. Strings which should be sent along with the request as metadata.
    :param gcp_conn_id: The connection ID to use when fetching connection info.
    :param impersonation_chain: Optional. Service account to impersonate using short-term
        credentials, or chained list of accounts required to get the access_token
        of the last account in the list, which will be impersonated in the request.
        If set as a string, the account must grant the originating account
        the Service Account Token Creator IAM role.
        If set as a sequence, the identities from the list must grant
        Service Account Token Creator IAM role to the directly preceding identity, with first
        account from the list granting this role to the originating account (templated).
    """

    template_fields: Sequence[str] = (
        "gcp_conn_id",
        "impersonation_chain",
        "google_ads_link_id",
        "property_id",
    )

    def __init__(
        self,
        *,
        property_id: str,
        google_ads_link_id: str,
        retry: Retry | _MethodDefault = DEFAULT,
        timeout: float | None = None,
        metadata: Sequence[tuple[str, str]] = (),
        gcp_conn_id: str = "google_cloud_default",
        impersonation_chain: str | Sequence[str] | None = None,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.property_id = property_id
        self.google_ads_link_id = google_ads_link_id
        self.retry = retry
        self.timeout = timeout
        self.metadata = metadata
        self.gcp_conn_id = gcp_conn_id
        self.impersonation_chain = impersonation_chain

    def execute(
        self,
        context: Context,
    ) -> Message:
        hook = GoogleAnalyticsAdminHook(
            gcp_conn_id=self.gcp_conn_id,
            impersonation_chain=self.impersonation_chain,
        )
        self.log.info(
            "Requesting the Google Ads link with id %s for the property_id %s",
            self.google_ads_link_id,
            self.property_id,
        )
        ads_links = hook.list_google_ads_links(
            property_id=self.property_id,
            retry=self.retry,
            timeout=self.timeout,
            metadata=self.metadata,
        )
        find_link = (item for item in ads_links if item.name.split("/")[-1] == self.google_ads_link_id)
        if ads_link := next(find_link, None):
            self.log.info("Successful request.")
            return GoogleAdsLink.to_dict(ads_link)
        raise AirflowNotFoundException(
            f"Google Ads Link with id {self.google_ads_link_id} and property id {self.property_id} not found"
        )



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

import pytest

from airflow.providers.google.cloud.hooks.gcs import GCSHook
from airflow.providers.google.cloud.transfers.salesforce_to_gcs import SalesforceToGcsOperator
from airflow.providers.salesforce.hooks.salesforce import SalesforceHook

TASK_ID = "test-task-id"
QUERY = "SELECT id, company FROM Lead WHERE company = 'Hello World Inc'"
SALESFORCE_CONNECTION_ID = "test-salesforce-connection"
GCS_BUCKET = "test-bucket"
GCS_OBJECT_PATH = "path/to/test-file-path"
EXPECTED_GCS_URI = f"gs://{GCS_BUCKET}/{GCS_OBJECT_PATH}"
GCP_CONNECTION_ID = "google_cloud_default"
SALESFORCE_RESPONSE = {
    "records": [
        {
            "attributes": {"type": "Lead", "url": "/services/data/v42.0/sobjects/Lead/00Q3t00001eJ7AnEAK"},
            "Id": "00Q3t00001eJ7AnEAK",
            "Company": "Hello World Inc",
        }
    ],
    "totalSize": 1,
    "done": True,
}
INCLUDE_DELETED = True
QUERY_PARAMS = {"DEFAULT_SETTING": "ENABLED"}


class TestSalesforceToGcsOperator:
    @pytest.mark.db_test
    @mock.patch.object(GCSHook, "upload")
    @mock.patch.object(SalesforceHook, "write_object_to_file")
    @mock.patch.object(SalesforceHook, "make_query")
    def test_execute(self, mock_make_query, mock_write_object_to_file, mock_upload):
        mock_make_query.return_value = SALESFORCE_RESPONSE

        operator = SalesforceToGcsOperator(
            query=QUERY,
            bucket_name=GCS_BUCKET,
            object_name=GCS_OBJECT_PATH,
            salesforce_conn_id=SALESFORCE_CONNECTION_ID,
            gcp_conn_id=GCP_CONNECTION_ID,
            include_deleted=INCLUDE_DELETED,
            query_params=QUERY_PARAMS,
            export_format="json",
            coerce_to_timestamp=True,
            record_time_added=True,
            task_id=TASK_ID,
        )
        result = operator.execute({})

        mock_make_query.assert_called_once_with(
            query=QUERY, include_deleted=INCLUDE_DELETED, query_params=QUERY_PARAMS
        )

        mock_write_object_to_file.assert_called_once_with(
            query_results=SALESFORCE_RESPONSE["records"],
            filename=mock.ANY,
            fmt="json",
            coerce_to_timestamp=True,
            record_time_added=True,
        )

        mock_upload.assert_called_once_with(
            bucket_name=GCS_BUCKET, object_name=GCS_OBJECT_PATH, filename=mock.ANY, gzip=False
        )

        assert result == EXPECTED_GCS_URI



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

from airflow.providers.google.marketing_platform.operators.display_video import (
    GoogleDisplayVideo360CreateSDFDownloadTaskOperator,
    GoogleDisplayVideo360SDFtoGCSOperator,
)
from airflow.utils import timezone

API_VERSION = "v4"
GCP_CONN_ID = "google_cloud_default"
IMPERSONATION_CHAIN = ["ACCOUNT_1", "ACCOUNT_2", "ACCOUNT_3"]

DEFAULT_DATE = timezone.datetime(2021, 1, 1)
REPORT_ID = "report_id"
BUCKET_NAME = "test_bucket"
REPORT_NAME = "test_report.csv"
QUERY_ID = FILENAME = "test.csv"
OBJECT_NAME = "object_name"
OPERATION_NAME = "test_operation"
RESOURCE_NAME = "resource/path/to/media"


class TestGoogleDisplayVideo360SDFtoGCSOperator:
    @mock.patch("airflow.providers.google.marketing_platform.operators.display_video.zipfile")
    @mock.patch("airflow.providers.google.marketing_platform.operators.display_video.os")
    @mock.patch(
        "airflow.providers.google.marketing_platform.operators.display_video.tempfile.TemporaryDirectory"
    )
    @mock.patch("airflow.providers.google.marketing_platform.operators.display_video.GCSHook")
    @mock.patch(
        "airflow.providers.google.marketing_platform.operators.display_video.GoogleDisplayVideo360Hook"
    )
    @mock.patch(
        "airflow.providers.google.marketing_platform.operators.display_video.open",
        new_callable=mock.mock_open,
    )
    def test_execute(self, mock_open, mock_hook, gcs_hook_mock, temp_dir_mock, os_mock, zipfile_mock):
        operation = {"response": {"resourceName": RESOURCE_NAME}}
        media = mock.Mock()

        mock_hook.return_value.get_sdf_download_operation.return_value = operation
        mock_hook.return_value.download_media.return_value = media

        tmp_dir = "/tmp/mock_dir"
        temp_dir_mock.return_value.__enter__.return_value = tmp_dir

        # Mock os behavior
        os_mock.path.join.side_effect = lambda *args: "/".join(args)
        os_mock.listdir.return_value = [FILENAME]

        # Mock zipfile behavior
        zipfile_mock.ZipFile.return_value.__enter__.return_value.extractall.return_value = None

        op = GoogleDisplayVideo360SDFtoGCSOperator(
            operation_name=OPERATION_NAME,
            bucket_name=BUCKET_NAME,
            object_name=OBJECT_NAME,
            gzip=False,
            api_version=API_VERSION,
            gcp_conn_id=GCP_CONN_ID,
            task_id="test_task",
            impersonation_chain=IMPERSONATION_CHAIN,
        )

        result = op.execute(context=None)

        # Assertions
        mock_hook.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        mock_hook.return_value.get_sdf_download_operation.assert_called_once_with(
            operation_name=OPERATION_NAME
        )
        mock_hook.return_value.download_media.assert_called_once_with(resource_name=RESOURCE_NAME)
        mock_hook.return_value.download_content_from_request.assert_called_once()

        gcs_hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        gcs_hook_mock.return_value.upload.assert_called_once_with(
            bucket_name=BUCKET_NAME,
            object_name=OBJECT_NAME,
            filename=f"{tmp_dir}/{FILENAME}",
            gzip=False,
        )

        assert result == f"{BUCKET_NAME}/{OBJECT_NAME}"


class TestGoogleDisplayVideo360CreateSDFDownloadTaskOperator:
    @mock.patch(
        "airflow.providers.google.marketing_platform.operators.display_video.GoogleDisplayVideo360Hook"
    )
    def test_execute(self, mock_hook):
        body_request = {
            "version": "1",
            "id": "id",
            "filter": {"id": []},
        }
        test_name = "test_task"

        # Create mock context with task_instance
        mock_context = {"task_instance": mock.Mock()}

        mock_hook.return_value.create_sdf_download_operation.return_value = {"name": test_name}

        op = GoogleDisplayVideo360CreateSDFDownloadTaskOperator(
            body_request=body_request,
            api_version=API_VERSION,
            gcp_conn_id=GCP_CONN_ID,
            task_id="test_task",
        )

        op.execute(context=mock_context)
        mock_hook.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=None,
        )

        mock_hook.return_value.create_sdf_download_operation.assert_called_once()
        mock_hook.return_value.create_sdf_download_operation.assert_called_once_with(
            body_request=body_request
        )
        mock_context["task_instance"].xcom_push.assert_called_once_with(key="name", value=test_name)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""This module contains Google Search Ads operators."""

from __future__ import annotations

from collections.abc import Sequence
from functools import cached_property
from typing import TYPE_CHECKING, Any

from airflow.providers.google.marketing_platform.hooks.search_ads import GoogleSearchAdsReportingHook
from airflow.providers.google.version_compat import BaseOperator

if TYPE_CHECKING:
    from airflow.providers.common.compat.sdk import Context


class _GoogleSearchAdsBaseOperator(BaseOperator):
    """
    Base class to use in NextGen operator.

    :param api_version: The version of the API that will be requested for example 'v0'.
    :param gcp_conn_id: The connection ID to use when fetching connection info.
    """

    template_fields: Sequence[str] = (
        "api_version",
        "gcp_conn_id",
    )

    def __init__(
        self,
        *,
        api_version: str = "v0",
        gcp_conn_id: str = "google_search_ads_default",
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.api_version = api_version
        self.gcp_conn_id = gcp_conn_id

    @cached_property
    def hook(self):
        return GoogleSearchAdsReportingHook(
            gcp_conn_id=self.gcp_conn_id,
            api_version=self.api_version,
        )


class GoogleSearchAdsSearchOperator(_GoogleSearchAdsBaseOperator):
    """
    Search a report by query.

    .. seealso:
        For API documentation check:
        https://developers.google.com/search-ads/reporting/api/reference/rest/v0/customers.searchAds360/search

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:GoogleSearchAdsSearchOperator`

    :param customer_id: The ID of the customer being queried.
    :param query: The query to execute.
    :param page_token: Token of the page to retrieve. If not specified, the first page of results will be
        returned. Use the value obtained from `next_page_token` in the previous response
        in order to request the next page of results.
    :param page_size: Number of elements to retrieve in a single page. When too large a page is requested,
        the server may decide to further limit the number of returned resources.
        Default is 10000.
    :param return_total_results_count: If true, the total number of results that match the query ignoring
        the LIMIT clause will be included in the response. Default is false.
    :param summary_row_setting: Determines whether a summary row will be returned. By default,
        summary row is not returned. If requested, the summary row will be sent
        in a response by itself after all others query results are returned.
    :param validate_only: If true, the request is validated but not executed. Default is false.
    :param gcp_conn_id: The connection ID to use when fetching connection info.
    :param api_version: The version of the API that will be requested for example 'v0'.
    """

    template_fields: Sequence[str] = (
        *_GoogleSearchAdsBaseOperator.template_fields,
        "page_token",
        "page_size",
    )

    def __init__(
        self,
        *,
        customer_id: str,
        query: str,
        page_token: str | None = None,
        page_size: int = 10000,
        return_total_results_count: bool = False,
        summary_row_setting: str | None = None,
        validate_only: bool = False,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.customer_id = customer_id
        self.query = query
        self.page_token = page_token
        self.page_size = page_size
        self.return_total_results_count = return_total_results_count
        self.summary_row_setting = summary_row_setting
        self.validate_only = validate_only

    def execute(self, context: Context):
        self.log.info("Querying Search Ads")
        response = self.hook.search(
            customer_id=self.customer_id,
            query=self.query,
            page_size=self.page_size,
            page_token=self.page_token,
            return_total_results_count=self.return_total_results_count,
            summary_row_setting=self.summary_row_setting,
            validate_only=self.validate_only,
        )
        self.log.info("Query result: %s", response)
        return response


class GoogleSearchAdsGetFieldOperator(_GoogleSearchAdsBaseOperator):
    """
    Retrieve metadata for a resource or a field.

    .. seealso:
        For API documentation check:
        https://developers.google.com/search-ads/reporting/api/reference/rest/v0/searchAds360Fields/get

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:GoogleSearchAdsGetFieldOperator`

    :param field_name: The name of the field.
    :param gcp_conn_id: The connection ID to use when fetching connection info.
    :param api_version: The version of the API that will be requested for example 'v0'.
    """

    def __init__(
        self,
        *,
        field_name: str,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.field_name = field_name

    def execute(self, context: Context) -> Any:
        self.log.info("Retrieving the metadata for the field '%s'", self.field_name)
        response = self.hook.get_field(field_name=self.field_name)
        self.log.info("Retrieved field: %s", response["resourceName"])
        return response


class GoogleSearchAdsSearchFieldsOperator(_GoogleSearchAdsBaseOperator):
    """
    Retrieve metadata for resource(s) or field(s) by the query syntax.

    .. seealso:
        For API documentation check:
        https://developers.google.com/search-ads/reporting/api/reference/rest/v0/searchAds360Fields/search

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:GoogleSearchAdsSearchFieldsOperator`

    :param query: The query string to execute.
    :param page_token: Token of the page to retrieve. If not specified, the first page of results will be
        returned. Use the value obtained from `next_page_token` in the previous response
        in order to request the next page of results.
    :param page_size: Number of elements to retrieve in a single page. When too large a page is requested,
        the server may decide to further limit the number of returned resources.
        Default 10000.
    :param gcp_conn_id: The connection ID to use when fetching connection info.
    :param api_version: The version of the API that will be requested for example 'v0'.
    """

    template_fields: Sequence[str] = (
        *_GoogleSearchAdsBaseOperator.template_fields,
        "page_token",
        "page_size",
    )

    def __init__(
        self,
        *,
        query: str,
        page_token: str | None = None,
        page_size: int = 10000,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.query = query

        self.page_token = page_token
        self.page_size = page_size

    def execute(self, context: Context) -> Any:
        self.log.info("Retrieving the metadata for %s", self.query)
        response = self.hook.search_fields(
            query=self.query,
            page_token=self.page_token,
            page_size=self.page_size,
        )
        self.log.info("Num of fields retrieved, #%d", len(response["results"]))
        return response


class GoogleSearchAdsGetCustomColumnOperator(_GoogleSearchAdsBaseOperator):
    """
    Retrieve details of a custom column for the given customer_id and campaign_id.

    .. seealso:
        For API documentation check:
        https://developers.google.com/search-ads/reporting/api/reference/rest/v0/customers.customColumns/get

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:GoogleSearchAdsGetCustomColumnOperator`

    :param customer_id: The customer ID for the custom column.
    :param custom_column_id: The ID for the custom column.
    :param gcp_conn_id: The connection ID to use when fetching connection info.
    :param api_version: The version of the API that will be requested for example 'v0'.
    """

    def __init__(
        self,
        *,
        customer_id: str,
        custom_column_id: str,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.customer_id = customer_id
        self.custom_column_id = custom_column_id

    def execute(self, context: Context):
        self.log.info(
            "Retrieving the custom column for the customer %s with the id of %s",
            self.customer_id,
            self.custom_column_id,
        )
        response = self.hook.get_custom_column(
            customer_id=self.customer_id,
            custom_column_id=self.custom_column_id,
        )
        self.log.info("Retrieved custom column: %s", response["id"])
        return response


class GoogleSearchAdsListCustomColumnsOperator(_GoogleSearchAdsBaseOperator):
    """
    List all custom columns.

    .. seealso:
        For API documentation check:
        https://developers.google.com/search-ads/reporting/api/reference/rest/v0/customers.customColumns/list

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:GoogleSearchAdsListCustomColumnsOperator`

    :param customer_id: The customer ID for the custom column.
    :param gcp_conn_id: The connection ID to use when fetching connection info.
    :param api_version: The version of the API that will be requested for example 'v0'.
    """

    def __init__(
        self,
        *,
        customer_id: str,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.customer_id = customer_id

    def execute(self, context: Context):
        self.log.info("Listing the custom columns for %s", self.customer_id)
        response = self.hook.list_custom_columns(customer_id=self.customer_id)
        self.log.info("Num of retrieved custom column: %d", len(response.get("customColumns")))
        return response



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""
Unit Tests for the Google Calendar Hook
"""

from __future__ import annotations

from unittest import mock

from airflow.providers.google.suite.hooks.calendar import GoogleCalendarHook

from unit.google.cloud.utils.base_gcp_mock import mock_base_gcp_hook_default_project_id

API_VERSION = "api_version"
GCP_CONN_ID = "test"
CALENDAR_ID = "test12345"
EVENT = {
    "summary": "Calendar Test Event",
    "description": "A chance to test creating an event from airflow.",
    "start": {
        "dateTime": "2021-12-28T09:00:00-07:00",
        "timeZone": "America/Los_Angeles",
    },
    "end": {
        "dateTime": "2021-12-28T17:00:00-07:00",
        "timeZone": "America/Los_Angeles",
    },
}
NUM_RETRIES = 5
API_RESPONSE = {"test": "response"}


class TestGoogleCalendarHook:
    def setup_method(self):
        with mock.patch(
            "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.__init__",
            new=mock_base_gcp_hook_default_project_id,
        ):
            self.hook = GoogleCalendarHook(api_version=API_VERSION, gcp_conn_id=GCP_CONN_ID)

    @mock.patch("airflow.providers.google.suite.hooks.calendar.GoogleCalendarHook.get_conn")
    def test_get_events(self, get_conn):
        get_method = get_conn.return_value.events.return_value.list
        execute_method = get_method.return_value.execute
        execute_method.return_value = {"kind": "calendar#events", "nextPageToken": None, "items": [EVENT]}
        result = self.hook.get_events(calendar_id=CALENDAR_ID)
        assert result == [EVENT]
        execute_method.assert_called_once_with(num_retries=NUM_RETRIES)
        get_method.assert_called_once_with(
            calendarId=CALENDAR_ID,
            iCalUID=None,
            maxAttendees=None,
            maxResults=None,
            orderBy=None,
            pageToken=None,
            privateExtendedProperty=None,
            q=None,
            sharedExtendedProperty=None,
            showDeleted=False,
            showHiddenInvitations=False,
            singleEvents=False,
            syncToken=None,
            timeMax=None,
            timeMin=None,
            timeZone=None,
            updatedMin=None,
        )

    @mock.patch("airflow.providers.google.suite.hooks.calendar.GoogleCalendarHook.get_conn")
    def test_create_event(self, mock_get_conn):
        create_mock = mock_get_conn.return_value.events.return_value.insert
        create_mock.return_value.execute.return_value = API_RESPONSE

        result = self.hook.create_event(calendar_id=CALENDAR_ID, event=EVENT)

        create_mock.assert_called_once_with(
            body=EVENT,
            calendarId=CALENDAR_ID,
            conferenceDataVersion=0,
            maxAttendees=None,
            sendNotifications=False,
            sendUpdates="false",
            supportsAttachments=False,
        )
        assert result == API_RESPONSE



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import asyncio
import logging
from unittest import mock

import pytest

from airflow.providers.google.cloud.triggers.mlengine import MLEngineStartTrainingJobTrigger
from airflow.triggers.base import TriggerEvent

TEST_CONN_ID = "ml_default"
TEST_JOB_ID = "1234"
TEST_GCP_PROJECT_ID = "test-project"
TEST_REGION = "us-central1"
TEST_RUNTIME_VERSION = "1.15"
TEST_PYTHON_VERSION = "3.9"
TEST_JOB_DIR = "gs://example_mlengine_bucket/job-dir"
TEST_PACKAGE_URIS = ["gs://system-tests-resources/example_gcp_mlengine/trainer-0.1.tar.gz"]
TEST_TRAINING_PYTHON_MODULE = "trainer.task"
TEST_TRAINING_ARGS: list[str] = []
TEST_LABELS = {"job_type": "training", "***-version": "v2-5-0-dev0"}
TEST_POLL_INTERVAL = 4.0


@pytest.fixture
def trigger():
    return MLEngineStartTrainingJobTrigger(
        conn_id=TEST_CONN_ID,
        job_id=TEST_JOB_ID,
        project_id=TEST_GCP_PROJECT_ID,
        region=TEST_REGION,
        runtime_version=TEST_RUNTIME_VERSION,
        python_version=TEST_PYTHON_VERSION,
        job_dir=TEST_JOB_DIR,
        package_uris=TEST_PACKAGE_URIS,
        training_python_module=TEST_TRAINING_PYTHON_MODULE,
        training_args=TEST_TRAINING_ARGS,
        labels=TEST_LABELS,
    )


class TestMLEngineStartTrainingJobTrigger:
    def test_serialize(self, trigger):
        """
        Asserts that the MLEngineStartTrainingJobTrigger correctly serializes its arguments
        and classpath.
        """

        classpath, kwargs = trigger.serialize()
        assert classpath == "airflow.providers.google.cloud.triggers.mlengine.MLEngineStartTrainingJobTrigger"
        assert kwargs == {
            "conn_id": TEST_CONN_ID,
            "job_id": TEST_JOB_ID,
            "project_id": TEST_GCP_PROJECT_ID,
            "region": TEST_REGION,
            "runtime_version": TEST_RUNTIME_VERSION,
            "python_version": TEST_PYTHON_VERSION,
            "job_dir": TEST_JOB_DIR,
            "poll_interval": TEST_POLL_INTERVAL,
            "package_uris": TEST_PACKAGE_URIS,
            "training_python_module": TEST_TRAINING_PYTHON_MODULE,
            "training_args": TEST_TRAINING_ARGS,
            "labels": TEST_LABELS,
        }

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.mlengine.MLEngineAsyncHook.get_job_status")
    async def test_trigger_on_success_yield_successfully(self, mock_job_status, trigger):
        """
        Tests the MLEngineStartTrainingJobTrigger only fires once the job execution reaches a successful state
        """
        mock_job_status.return_value = "success"

        generator = trigger.run()
        actual = await generator.asend(None)
        assert (
            TriggerEvent({"status": "success", "message": "Job completed", "job_id": TEST_JOB_ID}) == actual
        )

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.mlengine.MLEngineAsyncHook.get_job")
    async def test_trigger_on_running_wait_successfully(self, mocked_get, caplog, trigger):
        """
        Test that MLEngineStartTrainingJobTrigger does not fire while a job is still running.
        """

        mocked_get.side_effect = OSError()
        caplog.set_level(logging.INFO)

        task = asyncio.create_task(trigger.run().__anext__())
        await asyncio.sleep(0.5)

        # TriggerEvent was not returned
        assert task.done() is False

        assert "Job is still running..." in caplog.text
        assert f"Sleeping for {TEST_POLL_INTERVAL} seconds." in caplog.text

        # Prevents error when task is destroyed while in "pending" state
        asyncio.get_event_loop().stop()

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.mlengine.MLEngineAsyncHook.get_job_status")
    async def test_trigger_on_error_yield_successfully(self, mock_job_status, trigger):
        """
        Test that MLEngineStartTrainingJobTrigger fires the correct event in case of an error.
        """
        # Set the status to a value other than success or pending
        mock_job_status.return_value = "error"

        generator = trigger.run()
        actual = await generator.asend(None)
        assert TriggerEvent({"status": "error", "message": "error"}) == actual

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.google.cloud.hooks.mlengine.MLEngineAsyncHook.get_job_status")
    async def test_trigger_exec_yield_successfully(self, mock_job_status, trigger):
        """
        Test that MLEngineStartTrainingJobTrigger fires the correct event in case of an error.
        """
        mock_job_status.side_effect = Exception("Test exception")

        generator = trigger.run()
        actual = await generator.asend(None)
        assert TriggerEvent({"status": "error", "message": "Test exception"}) == actual



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

import pytest

from airflow.exceptions import AirflowException
from airflow.providers.google.cloud.hooks.gdm import GoogleDeploymentManagerHook


def mock_init(
    self,
    gcp_conn_id,
    impersonation_chain=None,
):
    pass


TEST_PROJECT = "my-project"
TEST_DEPLOYMENT = "my-deployment"


class TestDeploymentManagerHook:
    def setup_method(self):
        with mock.patch(
            "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.__init__",
            new=mock_init,
        ):
            self.gdm_hook = GoogleDeploymentManagerHook(gcp_conn_id="test")

    @mock.patch("airflow.providers.google.cloud.hooks.gdm.GoogleDeploymentManagerHook.get_conn")
    def test_list_deployments(self, mock_get_conn):
        response1 = {"deployments": [{"id": "deployment1", "name": "test-deploy1"}], "pageToken": None}
        response2 = {"deployments": [{"id": "deployment2", "name": "test-deploy2"}], "pageToken": None}

        mock_get_conn.return_value.deployments.return_value.list.return_value.execute.return_value = response1

        request_mock = mock.MagicMock()
        request_mock.execute.return_value = response2
        mock_get_conn.return_value.deployments.return_value.list_next.side_effect = [
            request_mock,
            None,
        ]

        deployments = self.gdm_hook.list_deployments(
            project_id=TEST_PROJECT, deployment_filter="filter", order_by="name"
        )

        mock_get_conn.assert_called_once_with()

        mock_get_conn.return_value.deployments.return_value.list.assert_called_once_with(
            project=TEST_PROJECT,
            filter="filter",
            orderBy="name",
        )

        assert mock_get_conn.return_value.deployments.return_value.list_next.call_count == 2

        assert deployments == [
            {"id": "deployment1", "name": "test-deploy1"},
            {"id": "deployment2", "name": "test-deploy2"},
        ]

    @mock.patch("airflow.providers.google.cloud.hooks.gdm.GoogleDeploymentManagerHook.get_conn")
    def test_delete_deployment(self, mock_get_conn):
        self.gdm_hook.delete_deployment(project_id=TEST_PROJECT, deployment=TEST_DEPLOYMENT)
        mock_get_conn.assert_called_once_with()
        mock_get_conn.return_value.deployments().delete.assert_called_once_with(
            project=TEST_PROJECT, deployment=TEST_DEPLOYMENT, deletePolicy=None
        )

    @mock.patch("airflow.providers.google.cloud.hooks.gdm.GoogleDeploymentManagerHook.get_conn")
    def test_delete_deployment_delete_fails(self, mock_get_conn):
        resp = {"error": {"errors": [{"message": "error deleting things.", "domain": "global"}]}}

        mock_get_conn.return_value.deployments.return_value.delete.return_value.execute.return_value = resp

        with pytest.raises(AirflowException):
            self.gdm_hook.delete_deployment(project_id=TEST_PROJECT, deployment=TEST_DEPLOYMENT)

        mock_get_conn.assert_called_once_with()
        mock_get_conn.return_value.deployments().delete.assert_called_once_with(
            project=TEST_PROJECT, deployment=TEST_DEPLOYMENT, deletePolicy=None
        )
